////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2008, Georgia Institute of Technology
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice,
//      this list of conditions and the following disclaimer.
//
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
//    * Neither the name of the Georgia Institute of Technology nor the names
//      of its contributors may be used to endorse or promote products derived
//      from this software without specific prior written permission.
//
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

#ifndef __ATSSCHEDULER_H__
#define __ATSSCHEDULER_H__

#include <iostream>
#include <sys/types.h>
#include <pthread.h>
#include <vector>

namespace wlpdstm
{
    class ATSScheduler
    {
    public:
	ATSScheduler( int threads);
	~ATSScheduler();

	void begin();
	
	void schedule( unsigned long id);
	void done( unsigned long id);

	static const double ALPHA = 0.3;	// weight for running avg.
	static const double THRESHOLD = 0.5;	// threshold for fallback

    private:
	std::vector<unsigned long> callers;
	int threads;
	bool* running;
	bool is_running;
	pthread_cond_t motherCond;
	pthread_mutex_t motherMutex;
	pthread_cond_t* conds;
	pthread_mutex_t* mutexes;
	 
	static const bool DEBUG = false;
    };
}

#endif // __ATSSCHEDULER_H__
