/**
 * This is not real getcpuid, but it guarantees that threads get different
 * integer ids in the range of 0..core_count-1 and that the id stays the same.
 */

#ifndef WLPDSTM_CPUID_H_
#define WLPDSTM_CPUID_H_

#include <unistd.h>

#include <pthread.h>

#include "tls.h"
#include "atomic.h"

namespace wlpdstm {

	class CpuId {
		private:
			static const int NO_ID = -1;

		public:
			CpuId() : cached_id(NO_ID) {
				// empty
			}

			void ThreadInit() {
				if(cached_id == NO_ID) {
					cached_id = GenerateCpuID();
				}
			}

			int GetCpuId() {
				return cached_id;
			}

			void SetCpuId(int id) {
				cached_id = id;
			}

			static void GlobalInit() {
				core_count = sysconf(_SC_NPROCESSORS_ONLN);
			}

		private:

			static int GenerateCpuID() {
				return fetch_and_inc_no_barrier(&current_id) % core_count;
			}

		private:
			int cached_id;

			static int current_id;

			static int core_count;
	};

	inline int sched_getcpu() {
		return Tls<CpuId, true, true>::Get()->GetCpuId();
		//return ::sched_getcpu();
	}
}



#endif /* WLPDSTM_CPUID_H_ */

